import unittest
import json
from etk.extractors.html_content_extractor import HTMLContentExtractor, Strategy


class TestGlossaryExtractor(unittest.TestCase):

    def test_html_content_extractor(self) -> None:
        hce = HTMLContentExtractor()

        with open('etk/unit_tests/ground_truth/sample_html.jl', 'r') as f:
            sample_html = json.load(f)

        test_html = sample_html["raw_content"]

        all_text = hce.extract(test_html, strategy=Strategy.ALL_TEXT)[0].value
        main_strict = hce.extract(test_html, strategy=Strategy.MAIN_CONTENT_STRICT)[0].value
        main_relax = hce.extract(test_html, strategy=Strategy.MAIN_CONTENT_RELAXED)[0].value

        expected_all_text = """         323-452-2013 Los Angeles Escort Listing View Escorts in other cities            smoothlegs24 28    chrissy391 27    My name is Helena height 160cms weight 55 kilos  contact me at escort.here@gmail.com           jefferson ave         age: 23 HrumpMeNow 28    xxtradition 24    jumblyjumb 26    claudia77 26    gushinPuss 28    Littlexdit 25    PinkSweets2 28    withoutlimit 27    bothOfUs3 28    lovelylips 27    killerbod 27    Littlexdit 27    azneyes 23      323-452-2013 ESCORT ALERT! - Luna The Hot Playmate (323) 452-2013 - 23  Escort's Phone:   323-452-2013    Escort's Location: Los Angeles, California  Escort's Age:  23  Date of Escort Post:  Jan 02nd 6:46am  REVIEWS:   READ AND CREATE REVIEWS FOR THIS ESCORT         There are 50 girls looking in . VIEW GIRLS  Hey I'm luna 3234522013 Let's explore , embrace and indulge in your favorite fantasy % independent. discreet no drama Firm Thighs and Sexy. My Soft skin & Tight Grip is exactly what you deserve Call or text Fetish friendly Fantasy friendly Party friendly 140 Hr SPECIALS 3234522013. Call 323-452-2013 . Me and my friends are on EZsex soooo you can find us all on there if you want... skittlegirl       Call me on my cell at 323-452-2013. Date of ad: 2017-01-02 06:46:00  More posts from 323-452-2013  323-452-2013  Nov 04, 2016 Guess whose New with 80Specials (323) 452-2013 - 21         Recent Escort Classifieds  North Jersey, New Jersey (732-621-4443)  :*: G O O D G I R L :*: G O N E *:**: B A D ;) LATINA - 21  Chicago, Illinois (773-412-2044)  ( LAtE NiGHt ) UNRUShEd (ULTiMAtE) PLEASURE (*AmAziNg Azz*) CHOOSE..W...  Chicago, Illinois (414-914-3777)  Petite. and Sweet. Super new and Ready... in out call - 21  Chicago, Illinois (312-600-8628)  CM Get PAMPERED - 22  Ft Lauderdale, Florida (786-504-1860)  you will ME. GORGEOUSLY sexy LATIN - 21  Atlanta, Georgia (401-324-9388)  WoW. MuSt TaKe A LoOk At ThIs. - 21  Atlanta, Georgia (347-940-1982)  SMOKiNG HOT Specials BuSTy BaBe (( 5 SeRvICe )) Pretty 36DDDs ( ) ( ...  Atlanta, Georgia (305-849-8140)  Here to please all your desiresCALL ME 100% Real Curvy White Girl. - 21  Houston, Texas (832-914-9667)  Beautiful Salvadorean The One And Only(: - 21  Phoenix, Arizona (623-500-7076)  NEW GIRL PERSIAN Gem EXotIC Blend - 21  Toronto, Ontario (416-554-3337)  (L) (L) ~~~Special 80 for 20 min:) 22YeAr oLd $$exyy LaTiNa BoMbSheLL~~(L...  Toronto, Ontario (416-520-5198)  **21 years old * $80 **real pictures ** A sian Kathy *** - 21  Toronto, Ontario (647-702-6825)  **: :** 18 Yr Old Mercedes **: :** Incalls/Outcalls (647)702-6825 - 1...  Toronto, Ontario (416-275-1296)  100% real pictures ** True college sweet heart ** Taiwan Kelly ** 416-275...  Northern Virginia, DC (206-922-9303)  SPECIALs 60/90/150 FuN & WiLd BrUnEtTe ThIcK & BuSty JUSt THe WAy U LIKe ...  Chicago, Illinois (608-609-5899)  New To Town Dont Miss Me 100$ Specials - 18  Dallas, Texas (469-510-5849)  JENNY ...A New RedHeaded Firecracker. YeeHaw. - 21  New York, New York (442-222-0227)  F R E A K Y __ O B S E S S I O N UPSCALE UNRUSHED-100%...  New York, New York (917-676-1333)  SuPeR SpEciALs Red BoNe ThiCk Incalls - 19  Houston, Texas (301-000-6018)  (Jamaican Ebony Goddess) petite sexy spontaneous UPSCALE discreet dates M...     Site Map  Call this Escort at 323-452-2013  View escorts in other cities  Register Here  Login to your account  Key for Escort Acronyms  Top 10 Escort Practices  Prostitution Laws Around the World  Prostitution Laws In Your State       Top Escort Cities  New York, New York  Toronto, Ontario  Dallas, Texas  Chicago, Illinois  Atlanta, Georgia  North Jersey, New Jersey  Detroit, Michigan  Los Angeles, California  San Diego, California  Orange County, California  Houston, Texas  Phoenix, Arizona  Philadelphia, Pennsylvania  Boston, Massachusetts  Washington DC, DC  Orlando, Florida  Las Vegas, Nevada  Miami, Florida     Recent Blog Posts  Sheriff candidate Minister and Detective Reno Fells arrested in prostitution bust Man gets 35 years for impersonating cop to get free sex from hooker Alexander Marino: Psychologist by Day, Pimp by Night Surfside Beach, SC Prostitution BUST: Video College Park, Maryland Prostitution Sting: 30 arrested SMU Players Robbed By Escort Study: 1 in 6 Sex Offenders Live Double Life Dark Knight Rises Suspect Killer James Holmes Bought Prostitutes Online Hookers Picked Up Off the Street Hookers paid extra for photos     Links  Escorts on Webcam  Ad Removal Form  Contact Us         Search Box   Search For Profiles   Register Here  Login to your account  Non Mobile Version  Escort Blog  Key for Escort Acronyms  Top 10 Escort Practices  Escort Reviews  See Escorts on Webcam  Prostitution Laws   Most Recently Viewed  Today at 10:02am Pacific      346-908-0622   Killeen      """
        # if readability is improved in the future, the expectation should be:
        # expected_main_strict = """\n \n \n \n \n \n smoothlegs24  28 \n \n \n chrissy391  27 \n \n \n My name is Helena height 160cms weight 55 kilos  contact me at escort.here@gmail.com           jefferson ave         age: 23 HrumpMeNow  28 \n \n \n xxtradition  24 \n \n \n jumblyjumb  26 \n \n \n claudia77  26 \n \n \n gushinPuss  28 \n \n \n Littlexdit  25 \n \n \n PinkSweets2  28 \n \n \n withoutlimit  27 \n \n \n bothOfUs3  28 \n \n \n lovelylips  27 \n \n \n killerbod  27 \n \n \n Littlexdit  27 \n \n \n azneyes  23 \n \n \n \n \n \n Escort's Phone: \n \n \n323-452-2013  \n \n Escort's Location: \nLos Angeles, California  \n Escort's Age:   23   Date of Escort Post:   Jan 02nd 6:46am \n REVIEWS:   \n READ AND CREATE REVIEWS FOR THIS ESCORT   \n \n \n \n \n \nThere are  50  girls looking in  .\n VIEW GIRLS \n \nHey I'm luna 3234522013 Let's explore , embrace and indulge in your favorite fantasy  % independent. discreet no drama Firm Thighs and Sexy. My Soft skin & Tight Grip is exactly what you deserve Call or text   Fetish friendly   Fantasy friendly   Party friendly 140 Hr SPECIALS 3234522013. Call  323-452-2013 .  Me and my friends are on EZsex  soooo you can find us all on there if you want... skittlegirl \n   \n \n   \n \n   \n Call me on my cell at 323-452-2013. Date of ad: 2017-01-02 06:46:00 \n \n \n"""
        # expected_main_relax = """\n \n \n \n \n \n \n smoothlegs24  28 \n \n \n chrissy391  27 \n \n \n My name is Helena height 160cms weight 55 kilos  contact me at escort.here@gmail.com           jefferson ave         age: 23 HrumpMeNow  28 \n \n \n xxtradition  24 \n \n \n jumblyjumb  26 \n \n \n claudia77  26 \n \n \n gushinPuss  28 \n \n \n Littlexdit  25 \n \n \n PinkSweets2  28 \n \n \n withoutlimit  27 \n \n \n bothOfUs3  28 \n \n \n lovelylips  27 \n \n \n killerbod  27 \n \n \n Littlexdit  27 \n \n \n azneyes  23 \n \n \n \n \n \n Escort's Phone: \n \n \n323-452-2013  \n \n Escort's Location: \nLos Angeles, California  \n Escort's Age:   23   Date of Escort Post:   Jan 02nd 6:46am \n REVIEWS:   \n READ AND CREATE REVIEWS FOR THIS ESCORT   \n \n \n \n \n \nThere are  50  girls looking in  .\n VIEW GIRLS \n \nHey I'm luna 3234522013 Let's explore , embrace and indulge in your favorite fantasy  % independent. discreet no drama Firm Thighs and Sexy. My Soft skin & Tight Grip is exactly what you deserve Call or text   Fetish friendly   Fantasy friendly   Party friendly 140 Hr SPECIALS 3234522013.\u00a0Call  323-452-2013 .  Me and my friends are on EZsex  soooo you can find us all on there if you want... skittlegirl \n \u00a0\u00a0\n \n \u00a0\u00a0\n \n \u00a0\u00a0\n Call me on my cell at 323-452-2013. Date of ad: 2017-01-02 06:46:00 \n \n \n \n" != "\n \n \n \n \n \n Escort's Phone: \n \n \n323-452-2013  \n \n Escort's Location: \nLos Angeles, California  \n Escort's Age:   23   Date of Escort Post:   Jan 02nd 6:46am \n REVIEWS:   \n READ AND CREATE REVIEWS FOR THIS ESCORT   \n \n \n \n \n \nThere are  50  girls looking in  .\n VIEW GIRLS \n \nHey I'm luna 3234522013 Let's explore , embrace and indulge in your favorite fantasy  % independent. discreet no drama Firm Thighs and Sexy. My Soft skin & Tight Grip is exactly what you deserve Call or text   Fetish friendly   Fantasy friendly   Party friendly 140 Hr SPECIALS 3234522013.\u00a0Call  323-452-2013 .  Me and my friends are on EZsex  soooo you can find us all on there if you want... skittlegirl \n \u00a0\u00a0\n \n \u00a0\u00a0\n \n \u00a0\u00a0\n Call me on my cell at 323-452-2013. Date of ad: 2017-01-02 06:46:00 \n \n \n \n"""
        expected_main_strict = """\n \n \n \n \n Escort's Phone: \n \n \n323-452-2013  \n \n Escort's Location: \nLos Angeles, California  \n Escort's Age:   23   Date of Escort Post:   Jan 02nd 6:46am \n REVIEWS:   \n READ AND CREATE REVIEWS FOR THIS ESCORT   \n \n \n \n \n \nThere are  50  girls looking in  .\n VIEW GIRLS \n \nHey I'm luna 3234522013 Let's explore , embrace and indulge in your favorite fantasy  % independent. discreet no drama Firm Thighs and Sexy. My Soft skin & Tight Grip is exactly what you deserve Call or text   Fetish friendly   Fantasy friendly   Party friendly 140 Hr SPECIALS 3234522013.\u00a0Call  323-452-2013 .  Me and my friends are on EZsex  soooo you can find us all on there if you want... skittlegirl \n \u00a0\u00a0\n \n \u00a0\u00a0\n \n \u00a0\u00a0\n Call me on my cell at 323-452-2013. Date of ad: 2017-01-02 06:46:00 \n \n \n"""
        expected_main_relax = """\n \n \n \n \n \n Escort's Phone: \n \n \n323-452-2013  \n \n Escort's Location: \nLos Angeles, California  \n Escort's Age:   23   Date of Escort Post:   Jan 02nd 6:46am \n REVIEWS:   \n READ AND CREATE REVIEWS FOR THIS ESCORT   \n \n \n \n \n \nThere are  50  girls looking in  .\n VIEW GIRLS \n \nHey I'm luna 3234522013 Let's explore , embrace and indulge in your favorite fantasy  % independent. discreet no drama Firm Thighs and Sexy. My Soft skin & Tight Grip is exactly what you deserve Call or text   Fetish friendly   Fantasy friendly   Party friendly 140 Hr SPECIALS 3234522013.\u00a0Call  323-452-2013 .  Me and my friends are on EZsex  soooo you can find us all on there if you want... skittlegirl \n \u00a0\u00a0\n \n \u00a0\u00a0\n \n \u00a0\u00a0\n Call me on my cell at 323-452-2013. Date of ad: 2017-01-02 06:46:00 \n \n \n \n"""

        self.assertEqual(json.dumps(all_text), json.dumps(expected_all_text))
        self.assertEqual(json.dumps(main_strict), json.dumps(expected_main_strict))
        self.assertEqual(json.dumps(main_relax), json.dumps(expected_main_relax))


if __name__ == '__main__':
    unittest.main()


from etk.document import Document
from etk.document_selector import DefaultDocumentSelector
from typing import List


class ETKModule(object):
    """
    Abstract class for extraction module
    """

    def __init__(self, etk):
        self.etk = etk

    def process_document(self, doc: Document) -> List[Document]:
        """
        Add your code for processing the document

        Returns:
            A list of new Document(s) created in this function, otherwise empty list()
        """
        pass

    def document_selector(self, doc: Document) -> bool:
        """
        Boolean function for selecting document
        Args:
            doc: Document

        Returns:

        """
        return DefaultDocumentSelector().select_document(doc)

    @property
    def produces(self) -> List[str]:
        return []

    @property
    def require(self) -> List[str]:
        return []



import unittest, datetime, pytz, json
from dateutil.relativedelta import relativedelta
from etk.extractors.date_extractor import DateExtractor, DateResolution
from etk.etk import ETK
from etk.knowledge_graph_schema import KGSchema


kg_schema = KGSchema(json.load(open('etk/unit_tests/ground_truth/test_config.json')))
de = DateExtractor(ETK(kg_schema=kg_schema), 'unit_test_date')


class TestDateExtractor(unittest.TestCase):
    # auxiliary method
    @staticmethod
    def convert_to_iso_format(date: datetime.datetime, resolution: DateResolution = DateResolution.DAY) -> str or None:
        """

        Args:
            date: datetime.datetime - datetime object to convert
            resolution: resolution of the iso format date to return

        Returns: string of iso format date

        """
        # TODO: currently the resolution is specified by the user, should it be decided what we have extracted,
        # E.g.: like if only year exists, use DateResolution.YEAR as resolution
        try:
            if date:
                date_str = date.isoformat()
                length = len(date_str)
                if resolution == DateResolution.YEAR and length >= 4:
                    return date_str[:4]
                elif resolution == DateResolution.MONTH and length >= 7:
                    return date_str[:7]
                elif resolution == DateResolution.DAY and length >= 10:
                    return date_str[:10]
                elif resolution == DateResolution.HOUR and length >= 13:
                    return date_str[:13]
                elif resolution == DateResolution.MINUTE and length >= 16:
                    return date_str[:16]
                elif resolution == DateResolution.SECOND and length >= 19:
                    return date_str[:19]
                return date_str
        except Exception as e:
            return None

        return None

    def test_ground_truth(self) -> None:
        with open('etk/unit_tests/ground_truth/date_ground_truth.txt', 'r') as f:
            texts = f.readlines()
        for text in texts:
            text = text.strip()
            if text and text[0] != '#':
                temp = text.split('|')
                if len(temp) == 3:
                    input_text, expected, format = temp
                    ignore_before = datetime.datetime(1890, 1, 1)
                    ignore_after = datetime.datetime(2500, 10, 10)
                    relative_base = datetime.datetime(2018, 1, 1)

                    e = de.extract(input_text,
                                   extract_first_date_only=False,
                                   additional_formats=[format],
                                   use_default_formats=False,
                                   ignore_dates_before=ignore_before,
                                   ignore_dates_after=ignore_after,
                                   detect_relative_dates=not format,
                                   relative_base=relative_base,
                                   preferred_date_order="DMY",
                                   prefer_language_date_order=True,
                                   return_as_timezone_aware=False,
                                   prefer_day_of_month='first',
                                   prefer_dates_from='current',
                                   date_value_resolution=DateResolution.SECOND
                                   if format and len(format) > 1 and format[1] in ['H', 'I'] else DateResolution.DAY
                                   )
                    expected = expected.replace('@today', self.convert_to_iso_format(datetime.datetime.now()))
                    if expected.startswith('@recentYear'):
                        today = datetime.datetime.now()
                        expected = expected.replace('@recentYear', str(today.year))
                        date = datetime.datetime.strptime(expected, '%Y-%m-%d')
                        next_year = date.replace(year=today.year+1)
                        last_year = date.replace(year=today.year-1)
                        if date > today and (date-today > today-last_year):
                            expected = str(last_year.year) + expected[4:]
                        elif date < today and (today-date > next_year-today):
                            expected = str(next_year.year) + expected[4:]
                    if expected and expected[0] != '@':
                        self.assertEqual(e[0].value if e else '', expected)

    def test_additional_formats(self) -> None:
        text = '2018@3@25  July 29 in 2018, 4/3@2018  2009-10-23 Jun 27 2017    D-3/8/91    S-07-08-2018'
        formats = ['%Y@%m@%d', '%B %d in %Y', '%m/%d@%Y', 'D-%d/%m/%y', 'S-%m-%d-%Y']

        extractions_with_default = de.extract(text, additional_formats=formats, use_default_formats=True )
        extractions_without_default = de.extract(text, additional_formats=formats, use_default_formats=False )

        results_with_default = [e.value for e in extractions_with_default]
        results_without_default = [e.value for e in extractions_without_default]

        expected_with_default = ['2018-03-25', '2018-07-29', '2018-04-03', '2009-10-23', '2017-06-27',
                                 '1991-08-03', '2018-07-08']
        expected_without_default = ['2018-03-25', '2018-07-29', '2018-04-03', '1991-08-03', '2018-07-08']

        self.assertEqual(results_with_default, expected_with_default)
        self.assertEqual(results_without_default, expected_without_default)

    def test_relative_date(self) -> None:
        text = '5 days ago, in two months, last year, yesterday, the day after tomorrow  ' \
               '2009-10-23 Jun 27 2017, what happened today'
        base = datetime.datetime(2018, 1, 1, tzinfo=pytz.timezone('UTC'))
        today = datetime.datetime.now()

        extractions_with_base = de.extract(text, detect_relative_dates=True, relative_base=base)
        extractions_base_tody = de.extract(text, detect_relative_dates=True)

        results_with_base = [e.value for e in extractions_with_base]
        results_base_today = [e.value for e in extractions_base_tody]

        relative = [relativedelta(days=-5), relativedelta(months=2), relativedelta(years=-1), relativedelta(days=-1),
                    relativedelta(days=2), relativedelta(days=0)]

        expected_with_base = ['2009-10-23', '2017-06-27'] + [self.convert_to_iso_format(base + x) for x in relative]
        expected_base_today = ['2009-10-23', '2017-06-27'] + [self.convert_to_iso_format(today + x) for x in relative]

        self.assertEqual(results_with_base, expected_with_base)
        self.assertEqual(results_base_today, expected_base_today)

    def test_order_preference(self) -> None:
        text = '10111211, 04/03/2018, 11121011'

        extractions_dmy = de.extract(text, preferred_date_order='DMY', prefer_language_date_order=False)
        extractions_mdy = de.extract(text, preferred_date_order='MDY', prefer_language_date_order=False)
        extractions_ymd = de.extract(text, preferred_date_order='YMD', prefer_language_date_order=False)

        results_dmy = [e.value for e in extractions_dmy]
        results_mdy = [e.value for e in extractions_mdy]
        results_ymd = [e.value for e in extractions_ymd]

        expected_dmy = ['1211-11-10', '2018-03-04', '1011-12-11']
        expected_mdy = ['1211-10-11', '2018-04-03', '1011-11-12']
        expected_ymd = ['1011-12-11', '2018-04-03', '1112-10-11']

        self.assertEqual(results_dmy, expected_dmy)
        self.assertEqual(results_mdy, expected_mdy)
        self.assertEqual(results_ymd, expected_ymd)

    def test_prefer_day(self) -> None:
        text = '2018 July and 09/20 and 2017/12'

        results = [
            [e.value for e in de.extract(text, prefer_day_of_month='first', preferred_date_order='DMY')],
            [e.value for e in de.extract(text, prefer_day_of_month='last', preferred_date_order='DMY')]
        ]
        expected = [['2018-07-01', '2020-09-01', '2017-12-01'], ['2018-07-31', '2020-09-30', '2017-12-31']]

        self.assertEqual(results, expected)

    def test_language(self) -> None:
        text = 'el 29 de febrero de 1996 vs lunes, el 24 de junio, 2013 vs 3 de octubre de 2017, and 04/03/2010'

        results = [
            [e.value for e in de.extract(text, prefer_language_date_order=True, preferred_date_order='MDY')],
            [e.value for e in de.extract(text, prefer_language_date_order=False, preferred_date_order='MDY')]
        ]

        expected = [
            ['1996-02-29', '2013-06-24', '2017-10-03', '2010-03-04'],
            ['1996-02-29', '2013-06-24', '2017-10-03', '2010-04-03']
        ]

        self.assertEqual(results, expected)

    def test_original_resolution(self) -> None:
        text = '2019-10-23 | 2017-06 | 2018-03-10 10:12 | July 2018 | Mar 2000 | year 2020'

        extractions = de.extract(text, date_value_resolution=DateResolution.ORIGINAL)

        results = [e.value for e in extractions]

        expected = ['2019-10-23', '2017-06', '2018-03-10T10:12', '2018-07', '2000-03', '2020']

        self.assertEqual(results, expected)

    def test_corner_cases(self) -> None:
        text='That star is MARS. ' \
             'It happened in 2012.' \
             'he may go to his hometown' \
             '13X or the ratio is 69/44  ' \
             'I was born in 94/10 ' \
             'I will take vocation on 12/23'

        extractions = de.extract(text=text, date_value_resolution=DateResolution.ORIGINAL, prefer_dates_from='future')

        results = [e.value for e in extractions]
        today = datetime.datetime.now()
        year = today.year
        if datetime.datetime(datetime.datetime.now().year, 12, 23) < today:
            year += 1
        expected = ['2012', '1994-10', '%d-12-23' % year]

        self.assertEqual(results, expected)


if __name__ == '__main__':
    unittest.main()



import unittest
from etk.extractors.sentence_extractor import SentenceExtractor

import spacy


class TestSentenceExtractor(unittest.TestCase):

    def setUp(self):
        self.sentence_extractor = SentenceExtractor()

    # Test1: simple split
    def test_simple_split(self):

        input_text = "Spacy's parser splits sentences better if the inputs " \
                     "are formatted with proper grammar. However, this is " \
                     "not always the case for text scraped from the internet."

        output_expected = ["Spacy's parser splits sentences better if the "
                           "inputs are formatted with proper grammar.",
                           "However, this is not always the case for text "
                           "scraped from the internet."]

        output_extractions = self.sentence_extractor.extract(input_text)
        for i, extraction in enumerate(output_extractions):
            self.assertEqual(extraction.value, output_expected[i])

    # Test2: ...end of sentence. start without caps
    def test_starts_lower(self):

        input_text = "Spacy's parser splits sentences better if the inputs " \
                     "are formatted with proper grammar. however, this is " \
                     "not always the case for text scraped from the internet."

        output_expected = ["Spacy's parser splits sentences better if the "
                           "inputs are formatted with proper grammar.",
                           "however, this is not always the case for text "
                           "scraped from the internet."]

        output_extractions = self.sentence_extractor.extract(input_text)
        for i, extraction in enumerate(output_extractions):
            self.assertEqual(extraction.value, output_expected[i])

    # Test3: ...end of sentence.Start without space
    def test_no_space(self):

        input_text = "Spacy's parser splits sentences better if the inputs " \
                     "are formatted with proper grammar.However, this is " \
                     "not always the case for text scraped from the internet."

        output_expected = ["Spacy's parser splits sentences better if the "
                           "inputs are formatted with proper grammar.",
                           "However, this is not always the case for text "
                           "scraped from the internet."]

        output_extractions = self.sentence_extractor.extract(input_text)
        for i, extraction in enumerate(output_extractions):
            self.assertEqual(extraction.value, output_expected[i])

    # Test4: ...end of sentence.worst case scenario
    def test_worst_case_scenario(self):

        input_text = "Spacy's parser splits sentences better if the inputs " \
                     "are formatted with proper grammar.however, this is " \
                     "not always the case for text scraped from the internet."

        output_expected = ["Spacy's parser splits sentences better if the "
                           "inputs are formatted with proper grammar."
                           "however, this is not always the case for text "
                           "scraped from the internet."]

        output_desired = ["Spacy's parser splits sentences better if the "
                          "inputs are formatted with proper grammar.",
                          "however, this is not always the case for text "
                          "scraped from the internet."]

        output_extractions = self.sentence_extractor.extract(input_text)

        for i, extraction in enumerate(output_extractions):
            self.assertEqual(extraction.value, output_expected[i])

        with self.assertRaises(AssertionError):
            for i, extraction in enumerate(output_extractions):
                self.assertEqual(extraction.value, output_desired[i])

    # Test5: load custom_nlp -> run 1-4
    def test_ze_custom_nlp(self):

        my_nlp = spacy.load("en_core_web_sm")
        self.sentence_extractor = SentenceExtractor(custom_nlp=my_nlp)

        self.test_simple_split()
        self.test_starts_lower()
        self.test_no_space()
        self.test_worst_case_scenario()


if __name__ == "__main__":
    unittest.main()



"""
Example CLI module
"""


def add_arguments(parser):
    """
    Parse arguments
    Args:
        parser (argparse.ArgumentParser)
    """
    parser.add_argument("-t", "--test", action="store", type=str, dest="test_string")


def run(args):
    """
    Args:
        args (argparse.Namespace)
    """
    print(args.test_string or 'no test string provided')





class OriginRecord():
    """
    An individual Origin Record of an extraction result from a document.
    """
    def __init__(self, json_path: str, start_char: str, end_char:str, document=None) -> None:
        # Extractable.__init__(self)
        self.json_path = json_path
        self.start_char = start_char
        self.end_char = end_char
        self._document = document

    @property
    def full_path(self) -> str:
        """
        Returns: The full path of a JSONPath match
        """
        return self.json_path

    @property
    def document(self):
        """
        Returns: the parent Document. It's prese
        """
        return self._document



import unittest, json
from etk.etk import ETK
from etk.knowledge_graph_schema import KGSchema

sample_input = {
    "projects": [
        {
            "name": "etk",
            "description": "version 2 of etk, implemented by Runqi, Dongyu, Sylvia, Amandeep and others."
        },
        {
            "name": "rltk",
            "description": "record linkage toolkit, implemented by Pedro, Mayank, Yixiang and several students."
        }
    ]
}


class TestSegment(unittest.TestCase):

    def test_segment(self) -> None:
        kg_schema = KGSchema(json.load(open('etk/unit_tests/ground_truth/test_config.json')))

        etk = ETK(kg_schema=kg_schema)
        doc = etk.create_document(sample_input)
        descriptions = doc.select_segments("projects[*].description")
        description_value = [i.value for i in descriptions]
        expected = [
            'version 2 of etk, implemented by Runqi, Dongyu, Sylvia, Amandeep and others.',
            'record linkage toolkit, implemented by Pedro, Mayank, Yixiang and several students.'
        ]
        self.assertEqual(description_value, expected)


if __name__ == '__main__':
    unittest.main()


import unittest
from etk.extractors.spacy_ner_extractor import SpacyNerExtractor
from etk.black_list_filter import BlackListFilter
from etk.extractors.glossary_extractor import GlossaryExtractor
from etk.tokenizer import Tokenizer


class TestBlackListFilter(unittest.TestCase):
    def setUp(self):
        self.text = 'Napoléon Bonaparte was a French statesman and military leader who rose to prominence during the ' \
                    'French Revolution and led several successful campaigns during the French Revolutionary Wars. ' \
                    'As Napoleon, he was Emperor of the French from 1804 until 1814, and again briefly in 1815 during ' \
                    'the Hundred Days. Napoleon dominated European and global affairs for more than a decade while ' \
                    'leading France against a series of coalitions in the Napoleonic Wars. He won most of these wars ' \
                    'and the vast majority of his battles, building a large empire that ruled over continental Europe ' \
                    'before its final collapse in 1815. He is considered one of the greatest commanders in history, ' \
                    'and his wars and campaigns are studied at military schools worldwide. Napoleon\'s political and ' \
                    'cultural legacy has endured as one of the most celebrated and controversial leaders in human history.'
        extractor = SpacyNerExtractor(extractor_name='spacy_ner_extractor')
        self.results = extractor.extract(self.text)

        glossary_1 = ['Beijing', 'Los Angeles', 'New York', 'Shanghai']
        t = Tokenizer()
        text = 'i live in los angeles. my hometown is Beijing. I love New York City.'
        tokens = t.tokenize(text)
        ge = GlossaryExtractor(glossary_1, 'test_glossary', t, 3, False)
        self.results2 = ge.extract(tokens)

    def test_black_list_filter(self) -> None:
        black_list = ["Napoléon Bonaparte", "France", "Napoleon"]
        blf = BlackListFilter(black_list)
        filtered_extractions = blf.filter(self.results)

        self.assertTrue(len(filtered_extractions) == 0)

    def test_black_list_filter_case_sensitive(self) -> None:
        black_list = ["Napoléon Bonaparte", "France", "napoleon"]
        blf = BlackListFilter(black_list)
        filtered_extractions = blf.filter(self.results, case_sensitive=True)
        self.assertTrue(len(filtered_extractions) == 2)

    def test_glossary_filter(self):
        black_list = ["los angeles"]
        blf = BlackListFilter(black_list)
        filtered_extractions = blf.filter(self.results2)
        self.assertTrue(len(filtered_extractions) == 2)


if __name__ == '__main__':
    unittest.main()



import unittest
from etk.extractors.language_identification_extractor import LanguageIdentificationExtractor


class TestLanguageIdentificationExtractor(unittest.TestCase):
    def test_langid(self):
        extractor = LanguageIdentificationExtractor()

        text_en = "langid.py comes pre-trained on 97 languages (ISO 639-1 codes given)"
        result_en = extractor.extract(text_en, "LANGID")
        self.assertEqual(result_en[0].value, "en")

        text_es = "Hola Mundo"
        result_es = extractor.extract(text_es, "LANGID")
        self.assertEqual(result_es[0].value, "es")

        text_de = "Ein, zwei, drei, vier"
        result_de = extractor.extract(text_de, "LANGID")
        self.assertEqual(result_de[0].value, "de")

        text_unknown = "%$@$%##"
        result_unknown = extractor.extract(text_unknown, "LANGID")
        self.assertEqual(result_unknown[0].value, "en")

    def test_langdetect(self):
        extractor = LanguageIdentificationExtractor()

        text_en = "langdetect supports 55 languages out of the box (ISO 639-1 codes)"
        result_en = extractor.extract(text_en, "LANGDETECT")
        self.assertEqual(result_en[0].value, "en")

        text_es = "Hola Mundo"
        result_es = extractor.extract(text_es, "LANGDETECT")
        self.assertEqual(result_es[0].value, "es")

        text_de = "Ein, zwei, drei, vier"
        result_de = extractor.extract(text_de, "LANGDETECT")
        self.assertEqual(result_de[0].value, "de")

        text_unknown = "%$@$%##"
        result_unknown = extractor.extract(text_unknown, "LANGDETECT")
        self.assertTrue(len(result_unknown) == 0)


if __name__ == '__main__':
    unittest.main()



from functools import reduce
from rdflib import Graph, URIRef
from rdflib.namespace import RDFS, OWL, SKOS, split_uri
from etk.ontology_api import Ontology, OntologyClass, OntologyProperty, OntologyObjectProperty
from etk.ontology_api import OntologyDatatypeProperty
from etk.ontology_namespacemanager import DIG, SCHEMA


class OntologyReportGenerator:
    def __init__(self, ontology: Ontology):
        self.ontology = ontology
        self.classes = ontology.classes
        self.properties = ontology.all_properties()
        self.data_properties = ontology.data_properties
        self.object_properties = ontology.object_properties
        self.item = '<h4 id="{}">{}</h4>\n<div class="entity">\n<table>\n{}\n</table>\n</div>'
        self.row = '<tr>\n<th align="right" valign="top">{}</th>\n<td>{}</td>\n</tr>'

    @staticmethod
    def sorted_name(arr):
        """
        Sorted input ontology entites by their names.

        :param arr: a set or list container of ontology entities
        :return: a sorted list
        """
        return sorted(arr, key=lambda x: x.name())

    def generate_html_report(self, include_turtle=False, exclude_warning=False, list_auxiliary_line=False) -> str:
        """
        Shows links to all classes and properties, a nice hierarchy of the classes, and then a nice
        description of all the classes with all the properties that apply to it.
        Example: http://www.cidoc-crm.org/sites/default/files/Documents/cidoc_crm_version_5.0.4.html

        :param include_turtle: include turtle related to this entity.
        :param exclude_warning: Exclude warning messages in HTML report
        :return: HTML in raw string
        """
        import os
        template = os.path.dirname(os.path.abspath(__file__)) + '/../ontologies/template.html'
        with open(template) as f:
            # Lists
            content = f.read().replace('{{{title}}}', 'Ontology Entities')
            content = content.replace('{{{class_list}}}', self.__html_entities_hierarchy(self.classes))
            content = content.replace('{{{dataproperty_list}}}', self.__html_entities_hierarchy(self.data_properties))
            content = content.replace('{{{objectproperty_list}}}', self.__html_entities_hierarchy(self.object_properties))
            # Classes
            content = content.replace('{{{classes}}}', self.__html_classes(include_turtle))
            # Properties
            properties = self.__html_properties(include_turtle)
            content = content.replace('{{{dataproperties}}}', properties[0])
            content = content.replace('{{{objectproperties}}}', properties[1])

            # Logging
            content = content.replace('{{{logging-title}}}', '' if exclude_warning else '<h2>Logging</h2>')
            logs = '' if exclude_warning else self.ontology.log_stream.getvalue()
            content = content.replace('{{{logging}}}', '<pre><code>{}</code></pre>'.format(logs))

            # Auxiliary line
            content = content.replace('{{{list_auxiliary_line}}}', self.__show_list_auxiliary_line(list_auxiliary_line))
        return content

    def __show_list_auxiliary_line(self, enable):
        return '''
            ul {
                /*list-style-type: none;*/
                margin-left: 3em;
                padding-left: 0em;
                list-style-position: inside;
                
            }
            
            ul > ul {
                border-left: 0.1em dotted #ddd;
            }
            
            ul > li {
                margin-left: -0.2em;
            }
        ''' if enable else ''

    def __html_classes(self, include_turtle):
        if not self.classes: return ''
        sorted_classes = self.sorted_name(self.classes)
        classes = []
        properties_map, referenced_map = self.__html_build_properties()
        inherit_properties, inherit_referenced = self.__html_build_inheritance_properties(properties_map,
                                                                                          referenced_map)
        for c in sorted_classes:
            attr = []
            subclass_of = self.sorted_name(c.super_classes())
            superclass_of = list(filter(lambda x: c in x.super_classes(), sorted_classes))
            if subclass_of:
                attr.append(self.row.format('Subclass of', '<br />\n'.join(map(self.__html_entity_href, subclass_of))))
            if superclass_of:
                attr.append(
                    self.row.format('Superclass of', '<br />\n'.join(map(self.__html_entity_href, superclass_of))))
            attr.extend(self.__html_entity_basic_info(c, self.row))
            properties = self.sorted_name(properties_map[c])
            if properties:
                attr.append(self.row.format('Properties', '<br />\n'.join(map(self.__html_class_property, properties))))
            properties = self.sorted_name(inherit_properties[c])
            if properties:
                attr.append(self.row.format('Properties (I)', '<br />\n'.join(map(self.__html_class_property, properties))))
            properties = self.sorted_name(referenced_map[c])
            if properties:
                attr.append(self.row.format('Referenced by', '<br />\n'.join(map(self.__html_class_referenced,
                                                                            properties))))
            properties = self.sorted_name(inherit_referenced[c])
            if properties:
                attr.append(self.row.format('Referenced (I)', '<br />\n'.join(map(self.__html_class_property, properties))))

            if include_turtle:
                code = self.__html_extract_other_info(c.uri())
                if code:
                    attr.append('<pre><code>{}</code></pre>'.format(code))
            classes.append(self.item.format('C-' + c.uri(), c.name(), '\n'.join(attr)))
        return '\n'.join(classes)
        
    def __html_properties(self, include_turtle):
        sorted_properties = self.sorted_name(self.properties)
        dataproperties, objectproperties = [], []
        for p in sorted_properties:
            attr = []
            domains = p.included_domains()
            ranges = p.included_ranges()
            if domains:
                attr.append(self.row.format('Domain', '<br />\n'.join(
                    map(self.__html_entity_href, self.sorted_name(domains)))))
            if ranges:
                if isinstance(p, OntologyObjectProperty):
                    attr.append(self.row.format('Range', '<br />\n'.join(
                        map(self.__html_entity_href, self.sorted_name(ranges)))))
                else:
                    ranges = map(self.qname, ranges)
                    attr.append(self.row.format('Range', '<br />\n'.join(sorted(ranges))))
            subproperty_of = self.sorted_name(p.super_properties())
            superproperty_of = list(filter(lambda x: p in x.super_properties(), sorted_properties))
            if subproperty_of:
                attr.append(self.row.format('Subproperty of', '<br />\n'.join(
                    map(self.__html_entity_href, subproperty_of))))
            if superproperty_of:
                attr.append(self.row.format('Superproperty of', '<br />\n'.join(
                    map(self.__html_entity_href, superproperty_of))))
            attr.extend(self.__html_entity_basic_info(p, self.row))
            if include_turtle:
                code = self.__html_extract_other_info(p.uri())
                if code:
                    attr.append('<pre><code>{}</code></pre>'.format(code))
            if isinstance(p, OntologyObjectProperty):
                if p.inverse():
                    attr.insert(0, self.row.format('Inverse', self.__html_entity_href(p.inverse())))
                objectproperties.append(self.item.format('O-' + p.uri(), p.name(), '\n'.join(attr)))
            else:
                dataproperties.append(self.item.format('D-' + p.uri(), p.name(), '\n'.join(attr)))
        return '\n'.join(dataproperties), '\n'.join(objectproperties)

    def __html_class_property(self, property_):
        range_ = property_.included_ranges()
        item = self.__html_entity_href(property_)
        if range_:
            tpl = ': [{}]' if len(range_) > 1 else ': {}'
            if isinstance(property_, OntologyObjectProperty):
                range_ = self.sorted_name(range_)
                item += tpl.format(', '.join(map(self.__html_entity_href, range_)))
            else:
                item += tpl.format(', '.join(map(self.qname, range_)))
        return item

    def qname(self, uri):
        namespace, _ = split_uri(uri)
        prefix = self.ontology.g.namespace_manager.store.prefix(URIRef(namespace))
        if prefix is None:
            return uri
        return self.ontology.g.qname(uri)

    def __html_class_referenced(self, property_):
        domains = self.sorted_name(property_.included_domains())
        item = self.__html_entity_href(property_)
        if domains:
            tpl = '[{}]: ' if len(domains) > 1 else '{} :'
            item = tpl.format(', '.join(map(self.__html_entity_href, domains))) + item
        return item

    def __html_build_properties(self):
        properties_map = {c: set() for c in self.classes}
        referenced_map = {c: set() for c in self.classes}
        for property_ in self.properties:
            for domain in property_.included_domains():
                properties_map[domain].add(property_)
            if isinstance(property_, OntologyObjectProperty):
                for range_ in property_.included_ranges():
                    referenced_map[range_].add(property_)
        return properties_map, referenced_map

    def __html_build_inheritance_properties(self, properties_map, referenced_map):
        properties = {k: set(v) for k, v in properties_map.items()}
        references = {k: set(v) for k, v in referenced_map.items()}
        tree, roots = self.__entity_tree(self.classes)
        while roots:
            for super_class in roots:
                for class_ in tree[super_class]:
                    properties[class_] |= properties[super_class]
                    references[class_] |= references[super_class]
            roots = reduce(set.union, (set(tree[super_class]) for super_class in roots))
        properties = {k: v-properties_map[k] for k, v in properties.items()}
        references = {k: v-referenced_map[k] for k, v in references.items()}
        return properties, references


    def __html_extract_other_info(self, uri):
        g = Graph()
        g.namespace_manager = self.ontology.g.namespace_manager
        if isinstance(uri, str):
            uri = URIRef(uri)
        for pred, obj in self.ontology.g.predicate_objects(uri):
            g.add((uri, pred, obj))
        g.remove((uri, DIG.commen_properties, None))
        g.remove((uri, SCHEMA.domainIncludes, None))
        g.remove((uri, SCHEMA.rangeIncludes, None))
        g.remove((uri, RDFS.domain, None))
        g.remove((uri, RDFS.range, None))
        g.remove((uri, RDFS.label, None))
        g.remove((uri, RDFS.comment, None))
        g.remove((uri, SKOS.definition, None))
        g.remove((uri, SKOS.note, None))
        g.remove((uri, OWL.subPropertyOf, None))
        for obj in self.ontology.g.objects(uri, RDFS.subClassOf):
            if isinstance(self.ontology.get_entity(uri), OntologyClass) and \
                    isinstance(self.ontology.get_entity(obj), OntologyClass):
                g.remove((uri, RDFS.subClassOf, obj))
        if len(g) < 2:
            return ''
        code = g.serialize(format='turtle').decode('utf-8').split('\n')
        code = filter(bool, code)
        code = filter(lambda line: line[0]!='@', code)
        return '\n'.join(code)

    @staticmethod
    def __html_entity_basic_info(e, tpl):
        attr = list()
        basic = ['label', 'definition', 'note', 'comment']
        for info in basic:
            attr.extend([tpl.format(info, item) for item in getattr(e, info)()])
        return attr

    @staticmethod
    def __entity_tree(entities):
        tree = {node: list() for node in entities}
        roots = []
        super = 'super_properties' if entities and isinstance(next(iter(entities)),
                                                              OntologyProperty) else 'super_classes'
        for child in entities:
            parents = getattr(child, super)()
            if not parents:
                roots.append(child)
            for parent in parents:
                tree[parent].append(child)
        return tree, roots

    def __html_entities_hierarchy(self, entities):
        tree, roots = self.__entity_tree(entities)

        def hierarchy_builder(children):
            if not children: return ''
            s = '<ul>\n'
            for child in sorted(children, key=lambda x: x.name()):
                s += '<li>{}</li>\n'.format(self.__html_entity_href(child))
                s += hierarchy_builder(tree[child])
            s += '</ul>\n'
            return s

        return hierarchy_builder(roots)

    @staticmethod
    def __html_entity_href(e):
        template = '<a href="#{}-{}">{}</a>'
        kind = {OntologyClass: 'C', OntologyObjectProperty: 'O', OntologyDatatypeProperty: 'D'}
        return template.format(kind[type(e)], e.uri(), e.name())




from enum import Enum, auto


class FieldType(Enum):
    """
    TEXT: value must be a string
    NUMBER: value must be a number
    LOCATION: value must be a location format
    DATE: value must be a date format
    """
    STRING = auto()
    NUMBER = auto()
    LOCATION = auto()
    DATE = auto()
    KG_ID = auto()



from enum import Enum, auto
from typing import List
from etk.extraction import Extraction


class InputType(Enum):
    """
    TEXT: value must be a string
    HTML: the value is HTML text
    TOKENS: value must first be tokenized
    OBJECT: value can be anything
    """
    TEXT = auto()
    HTML = auto()
    TOKENS = auto()
    OBJECT = auto()


class Extractor(object):
    """
    All extractors extend this abstract class.
    """

    def __init__(self, input_type: InputType = InputType.TEXT, category: str = None, name: str = None):
        self._category = category
        self._name = name
        self._input_type = input_type

    @property
    def input_type(self) -> InputType:
        """
        The type of input that an extractor wants
        Returns: InputType
        """
        return self._input_type

    @property
    def name(self) -> str:
        """
        The name of an extractor shown to users.
        Different instances ought to have different names, e.g., a glossary extractor for city_name could have
        name "city name extractor".

        Returns: string, the name of an extractor.
        """
        return self._name

    @property
    def category(self) -> str:
        """
        Identifies a whole category of extractors, all instances should have the same category and
        different names.

        Returns: string, a label to identify the category of an extractor.
        """
        return self._category

    def extract(self, *input_value, **configs) -> List[Extraction]:
        """

        Args:
            input_value (): some extractors may want multiple arguments, for example, to
            concatenate them together
            configs (): any configs/options of extractors

        Returns: list of extracted data, which can be any Python object
        """
        pass



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open('requirements.txt', 'r') as f:
    install_requires = list()
    for line in f:
        re = line.strip()
        if re:
            install_requires.append(re)

setuptools.setup(
    name="etk",
    version="2.1.1",
    author="Amandeep Singh",
    author_email="amandeep.s.saggu@gmail.com",
    description="extraction toolkit",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/usc-isi-i2/etk",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ),
    install_requires=install_requires
)



import unittest, json
from etk.extractors.inferlink_extractor import InferlinkExtractor, InferlinkRuleSet


class TestInferlinkExtractor(unittest.TestCase):

    def test_inferlink_extractor(self) -> None:
        with open('etk/unit_tests/ground_truth/sample_html.jl', 'r') as f:
            sample_html = json.load(f)['raw_content']
        rules = InferlinkRuleSet(InferlinkRuleSet.load_rules_file('etk/unit_tests/ground_truth/sample_inferlink_rules.json'))

        e = InferlinkExtractor(rules)
        test_result = e.extract(sample_html)

        result = [x.value for x in test_result]

        expected = [
            "323-452-2013",
            "Los Angeles, California",
            "23",
            "Hey I'm luna 3234522013 Let's explore , embrace and indulge in your favorite fantasy % independent. "
            "discreet no drama Firm Thighs and Sexy. My Soft skin & Tight Grip is exactly what you deserve Call or "
            "text Fetish friendly Fantasy friendly Party friendly 140 Hr SPECIALS 3234522013",
            "2017-01-02 06:46"
        ]

        self.assertEqual(result, expected)


if __name__ == '__main__':
    unittest.main()


from etk.extractors.regex_extractor import RegexExtractor


class IPAddressExtractor(RegexExtractor):
    """
    **Description**
           This class inherits RegexExtractor and predefines the ip address pattern

    Examples:
        ::

            ip_address_extractor = IPAddressExtractor()
            ip_address_extractor.extract(text=input_doc)
    """
    def __init__(self):
        ip_address_pattern = r"(?:(?:[01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])" \
                             r"[ (?:\[]?(?:\.|dot)[ )\]]?){3}(?:[01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])"
        RegexExtractor.__init__(self, pattern=ip_address_pattern, extractor_name="ip address extractor")



import etk.timeseries.extract_spreadsheet as extractSpreadsheet
from typing import List
from etk.document import Document
from etk.etk_exceptions import InvalidArgumentsError


class TimeseriesProcessor(object):
    def __init__(self, etk, **mapping_spec) -> None:
        self.etk = etk
        # < annotation > < spreadsheet > < output >
        if mapping_spec['annotation'] is None:
            #  TODO: adding auto infer annotation code
            pass
        else:
            self.annotation = mapping_spec['annotation']

        if mapping_spec['spreadsheet'] is None:
            raise InvalidArgumentsError("for argument 'spreadsheet', please specify spreadsheet path")
            # print("Please specify spreadsheet path")
        else:
            self.spreadsheet = mapping_spec['spreadsheet']

    def timeseries_extractor(self, file_name: str = None,
                             data_set: str = None) -> List[Document]:
        es = extractSpreadsheet.ExtractSpreadsheet(self.spreadsheet, self.annotation)
        timeseries = es.process()

        return self.create_documents(json_objs=timeseries, file_name=file_name, data_set=data_set)

    def create_documents(self, json_objs: object,
                         file_name: str = None,
                         data_set: str = None, ) -> List[Document]:
        documents = list()

        for json_obj in json_objs:
            cdr_doc = json_obj

            if file_name is not None:
                cdr_doc['file_name'] = file_name
            if data_set is not None:
                cdr_doc['data_set'] = data_set

            documents.append(self.etk.create_document(cdr_doc))

        return documents



import spacy
import json
import pandas as pd
import re
from etk.document import Document


class DocRetrieveProcessor(object):

    def __init__(self, etk, ifp_id: str, ifp_title: str, orig_ifp_title: str) -> None:
        self.etk = etk
        self.ifp_id = ifp_id
        self.ifp_title = ifp_title
        self.nlp = spacy.load('en_core_web_lg')
        self.query_tokens = self.nlp(self.ifp_title)
        self.orig_ifp_title = orig_ifp_title

        sentence_tokenizer = '(?<!\w\.\w)(?<![A-Z][a-z]\.)(?<=\.|\?)\s+(?=[A-Z])'
        self.sentence_tokenizer_pattern = re.compile(sentence_tokenizer)

    # process document sentence by sentence
    def process_by_sentence(self, doc: Document, threshold: float) -> Document:
        json_obj = doc.cdr_document

        # secondary key
        timestamp = json_obj['@timestamp']
        date = pd.to_datetime(timestamp, infer_datetime_format=True)
        doc_id = json_obj['doc_id']
        content = json_obj['lexisnexis']['doc_description'].replace('\n', " ").replace("[\\t\\n\\r]+", " ")
        sentences = self.sentence_tokenizer_pattern.split(content)

        scores = list(map(self.query_tokens.similarity, list(map(self.nlp, sentences))))
        # Note: ^ this is returning the cosSim of the AVERAGED .vectors in the list of tokens being compared
        max_score = max(scores)
        max_score_idx = scores.index(max_score)
        max_score_sentence = sentences[max_score_idx]

        if max_score > threshold:
            output_cdr_doc = {
                'type': 'News/IPF Relevance',
                'date': timestamp,
                'ifp': self.orig_ifp_title,
                'news_story': doc_id,
                'similarity': max_score,
                'matched_sentence': max_score_sentence
            }
        else:
            output_cdr_doc = {
                'type': None,
                'date': None,
                'ifp': None,
                'news_story': None,
                'similarity': 0.0,
                'matched_sentence': None
            }

        return self.etk.create_document(output_cdr_doc)

    def process_by_title(self, doc: Document, threshold: float) -> Document:
        json_obj = doc.cdr_document

        # secondary key
        timestamp = json_obj['@timestamp']
        date = pd.to_datetime(timestamp, infer_datetime_format=True)
        doc_id = json_obj['doc_id']

        title = json_obj['lexisnexis']['doc_title']
        title_token = self.nlp(title)
        similarity = self.query_tokens.similarity(title_token)
        # Note: ^ this is returning the cosSim of the AVERAGED .vectors in the list of tokens being compared

        if similarity > threshold:
            output_obj = {
                'type': 'News/IPF Relevance',
                'date': timestamp,
                'ifp': self.orig_ifp_title,
                'news_story': doc_id,
                'title': title,
                'similarity': similarity
            }
        else:
            output_obj = {
                'type': None,
                'date': None,
                'ifp': None,
                'news_story': None,
                'title': None,
                'similarity': 0.0
            }

        return self.etk.create_document(output_obj)